import tkinter as tk
from tkinter import ttk, filedialog, messagebox
import os
import json
import shutil
import zipfile
import datetime
import uuid

# --- הגדרות קבועות ---
MANIFEST_FILENAME = "manifest.json"
APKS_FOLDER_IN_ZIP = "apks"
IMAGES_FOLDER_IN_ZIP = "images"
OUTPUT_ZIP_FILENAME_PREFIX = "KosherStoreBackup_"

DEFAULT_PREFERENCES = {
    "settingsPassword": "yossi6547",
    "installationMode": "normal",
    "rootInstallEnabled": True,
    "rootInstallMethod": "pm_install",
    "copyToSystemPath": "/system/app",
    "showFeedbackMenu": True,
    "aboutText": "<p>זוהי חנות אפליקציות כשרה שנוצרה באמצעות סקריפט GUI.</p>"
}

# --- פונקציות עזר ---
def sanitize_filename(filename):
    return "".join(c if c.isalnum() or c in ['.', '_', '-'] else '_' for c in filename).replace(' ', '_')

class AppEntry:
    def __init__(self, master_frame, apk_path):
        self.apk_path = apk_path
        self.apk_basename = os.path.basename(apk_path)
        
        self.frame = ttk.LabelFrame(master_frame, text=f"APK: {self.apk_basename}", padding=(10, 5))
        self.frame.pack(fill=tk.X, padx=5, pady=5)

        ttk.Label(self.frame, text="שם אפליקציה:").grid(row=0, column=0, padx=5, pady=2, sticky=tk.W)
        self.name_var = tk.StringVar(value=os.path.splitext(self.apk_basename)[0])
        ttk.Entry(self.frame, textvariable=self.name_var, width=40).grid(row=0, column=1, padx=5, pady=2, sticky=tk.EW)

        ttk.Label(self.frame, text="תיאור:").grid(row=1, column=0, padx=5, pady=2, sticky=tk.W)
        self.desc_var = tk.StringVar()
        ttk.Entry(self.frame, textvariable=self.desc_var, width=40).grid(row=1, column=1, padx=5, pady=2, sticky=tk.EW)

        ttk.Label(self.frame, text="שם חבילה*:").grid(row=2, column=0, padx=5, pady=2, sticky=tk.W)
        self.package_var = tk.StringVar()
        self.package_entry = ttk.Entry(self.frame, textvariable=self.package_var, width=40)
        self.package_entry.grid(row=2, column=1, padx=5, pady=2, sticky=tk.EW)

        ttk.Label(self.frame, text="קובץ תמונה (אופציונלי):").grid(row=3, column=0, padx=5, pady=2, sticky=tk.W)
        self.image_path_var = tk.StringVar()
        ttk.Entry(self.frame, textvariable=self.image_path_var, width=30, state="readonly").grid(row=3, column=1, padx=5, pady=2, sticky=tk.EW)
        ttk.Button(self.frame, text="בחר תמונה...", command=self.select_image).grid(row=3, column=2, padx=5, pady=2)
        
        # נסה למצוא תמונה משויכת אוטומטית
        self.auto_find_associated_image()

    def select_image(self):
        filepath = filedialog.askopenfilename(
            title="בחר קובץ תמונה",
            filetypes=(("Image files", "*.png *.jpg *.jpeg"), ("All files", "*.*"))
        )
        if filepath:
            self.image_path_var.set(filepath)
            
    def auto_find_associated_image(self):
        base, _ = os.path.splitext(self.apk_path)
        for ext in ['.png', '.jpg', '.jpeg']:
            image_path = base + ext
            if os.path.exists(image_path):
                self.image_path_var.set(image_path)
                break

    def get_data(self):
        if not self.package_var.get().strip():
            messagebox.showerror("שגיאה", f"שם חבילה חסר עבור APK: {self.apk_basename}")
            self.package_entry.focus_set()
            return None
            
        return {
            "name": self.name_var.get().strip(),
            "description": self.desc_var.get().strip(),
            "packageName": self.package_var.get().strip(),
            "original_apk_path": self.apk_path,
            "original_image_path": self.image_path_var.get().strip() or None
        }

class AppBackupCreatorGUI:
    def __init__(self, root):
        self.root = root
        self.root.title("יוצר גיבוי לחנות האפליקציות")
        self.root.geometry("600x700")

        self.app_entries = []
        self.preferences = DEFAULT_PREFERENCES.copy()

        # --- מסגרת ראשית ---
        main_frame = ttk.Frame(root, padding="10")
        main_frame.pack(expand=True, fill=tk.BOTH)

        # --- בחירת קבצי APK ---
        apk_select_frame = ttk.Frame(main_frame)
        apk_select_frame.pack(fill=tk.X, pady=5)
        ttk.Button(apk_select_frame, text="בחר קבצי APK...", command=self.select_apks).pack(side=tk.LEFT, padx=5)
        self.selected_apks_label = ttk.Label(apk_select_frame, text="לא נבחרו קבצי APK")
        self.selected_apks_label.pack(side=tk.LEFT, padx=5)

        # --- אזור גלילה לפרטי האפליקציות ---
        canvas = tk.Canvas(main_frame)
        scrollbar = ttk.Scrollbar(main_frame, orient="vertical", command=canvas.yview)
        self.scrollable_frame = ttk.Frame(canvas)

        self.scrollable_frame.bind(
            "<Configure>",
            lambda e: canvas.configure(scrollregion=canvas.bbox("all"))
        )
        canvas.create_window((0, 0), window=self.scrollable_frame, anchor="nw")
        canvas.configure(yscrollcommand=scrollbar.set)

        canvas.pack(side="left", fill="both", expand=True, pady=5)
        scrollbar.pack(side="right", fill="y")
        
        # --- הגדרות ---
        ttk.Button(main_frame, text="ערוך הגדרות חנות...", command=self.edit_preferences).pack(pady=10)


        # --- יצירת גיבוי ---
        ttk.Button(main_frame, text="צור קובץ גיבוי ZIP...", command=self.create_backup).pack(pady=20)

    def select_apks(self):
        filepaths = filedialog.askopenfilenames(
            title="בחר קבצי APK (אפשר לבחור מספר קבצים)",
            filetypes=(("APK files", "*.apk"), ("All files", "*.*"))
        )
        if filepaths:
            self.selected_apks_label.config(text=f"נבחרו {len(filepaths)} קבצי APK")
            # נקה רשומות קודמות
            for entry in self.app_entries:
                entry.frame.destroy()
            self.app_entries.clear()

            # צור רשומות חדשות
            for apk_path in filepaths:
                app_entry = AppEntry(self.scrollable_frame, apk_path)
                self.app_entries.append(app_entry)
    
    def edit_preferences(self):
        pref_window = tk.Toplevel(self.root)
        pref_window.title("עריכת הגדרות החנות")
        pref_window.geometry("400x300")
        
        entries = {}
        row_idx = 0
        for key, value in self.preferences.items():
            ttk.Label(pref_window, text=f"{key}:").grid(row=row_idx, column=0, padx=5, pady=2, sticky=tk.W)
            var = tk.StringVar(value=str(value)) # המרה לסטרינג לצורך Entry
            entry = ttk.Entry(pref_window, textvariable=var, width=30)
            entry.grid(row=row_idx, column=1, padx=5, pady=2, sticky=tk.EW)
            entries[key] = (var, type(value)) # שמור גם את הטיפוס המקורי
            row_idx += 1
            
        def save_prefs():
            try:
                for key, (var, original_type) in entries.items():
                    val_str = var.get()
                    if original_type == bool:
                        self.preferences[key] = val_str.lower() in ['true', '1', 'yes']
                    elif original_type == int:
                        self.preferences[key] = int(val_str)
                    elif original_type == float: # אם יש צורך
                         self.preferences[key] = float(val_str)
                    else: # string
                        self.preferences[key] = val_str
                messagebox.showinfo("הגדרות", "ההגדרות נשמרו (עבור הגיבוי הנוכחי)", parent=pref_window)
                pref_window.destroy()
            except ValueError as e:
                messagebox.showerror("שגיאה", f"ערך לא תקין: {e}", parent=pref_window)

        ttk.Button(pref_window, text="שמור הגדרות", command=save_prefs).grid(row=row_idx, column=0, columnspan=2, pady=10)


    def create_backup(self):
        apps_data_to_include = []
        for entry in self.app_entries:
            data = entry.get_data()
            if data is None: # אם הייתה שגיאה (למשל, חסר שם חבילה)
                return 
            apps_data_to_include.append(data)

        if not apps_data_to_include:
            messagebox.showwarning("אין אפליקציות", "אנא בחר לפחות קובץ APK אחד והזן את פרטיו.")
            return

        output_zip_path = filedialog.asksaveasfilename(
            title="שמור קובץ גיבוי ZIP",
            defaultextension=".zip",
            initialfile=f"{OUTPUT_ZIP_FILENAME_PREFIX}{datetime.datetime.now().strftime('%Y%m%d_%H%M%S')}.zip",
            filetypes=(("ZIP files", "*.zip"), ("All files", "*.*"))
        )

        if not output_zip_path:
            return # המשתמש ביטל

        try:
            self.perform_zip_creation(output_zip_path, apps_data_to_include, self.preferences)
            messagebox.showinfo("הצלחה", f"קובץ הגיבוי נוצר בהצלחה:\n{output_zip_path}")
        except Exception as e:
            messagebox.showerror("שגיאה ביצירת ZIP", f"אירעה שגיאה: {e}")
            print(f"שגיאה מפורטת ביצירת ZIP: {e}") # לדיבוג

    def perform_zip_creation(self, zip_filepath, apps_to_include, preferences):
        app_list_for_manifest = []
        
        # תיקיית עבודה זמנית שתימחק בסוף
        temp_work_dir = f"temp_backup_work_{uuid.uuid4()}"
        os.makedirs(os.path.join(temp_work_dir, APKS_FOLDER_IN_ZIP), exist_ok=True)
        os.makedirs(os.path.join(temp_work_dir, IMAGES_FOLDER_IN_ZIP), exist_ok=True)

        timestamp_prefix = datetime.datetime.now().strftime("%Y%m%d%H%M%S")

        for idx, app_data in enumerate(apps_to_include):
            original_apk_path = app_data['original_apk_path']
            apk_basename = os.path.basename(original_apk_path)
            
            internal_apk_filename = f"apk_{timestamp_prefix}_{idx}_{sanitize_filename(apk_basename)}"
            
            app_info_for_manifest = {
                "name": app_data["name"],
                "description": app_data["description"],
                "packageName": app_data["packageName"],
                "apkFileName": internal_apk_filename,
                "imageFileName": None,
                "wasInstalledAsSystemByStore": False
            }

            dest_apk_path_in_temp = os.path.join(temp_work_dir, APKS_FOLDER_IN_ZIP, internal_apk_filename)
            shutil.copy2(original_apk_path, dest_apk_path_in_temp)

            if app_data.get('original_image_path'):
                original_image_path = app_data['original_image_path']
                image_basename = os.path.basename(original_image_path)
                _, image_ext = os.path.splitext(image_basename)
                internal_image_filename = f"img_{timestamp_prefix}_{idx}_{sanitize_filename(os.path.splitext(image_basename)[0])}{image_ext}"
                
                dest_image_path_in_temp = os.path.join(temp_work_dir, IMAGES_FOLDER_IN_ZIP, internal_image_filename)
                shutil.copy2(original_image_path, dest_image_path_in_temp)
                app_info_for_manifest["imageFileName"] = internal_image_filename
            
            app_list_for_manifest.append(app_info_for_manifest)

        manifest_data = {
            "preferences": preferences,
            "appList": app_list_for_manifest
        }
        manifest_temp_path = os.path.join(temp_work_dir, MANIFEST_FILENAME)
        with open(manifest_temp_path, 'w', encoding='utf-8') as f:
            json.dump(manifest_data, f, ensure_ascii=False, indent=4)

        with zipfile.ZipFile(zip_filepath, 'w', zipfile.ZIP_DEFLATED) as zf:
            for root, _, files in os.walk(temp_work_dir):
                for file in files:
                    file_path = os.path.join(root, file)
                    archive_name = os.path.relpath(file_path, temp_work_dir)
                    zf.write(file_path, archive_name)
        
        shutil.rmtree(temp_work_dir) # נקה תיקייה זמנית


if __name__ == "__main__":
    root = tk.Tk()
    app_gui = AppBackupCreatorGUI(root)
    root.mainloop()